import java.awt.*;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Vector;

import javax.swing.*;
import javax.swing.table.*;

import snaptaxi.*;

/**
 * @author Korea Univ. SE Team #1
 * @version 1.0
 * @since 2004.11.14
 * [2004.11.14.] ~ Add source.
 */
public class Server implements WindowListener, ServerListener
{
	private JFrame window = null;
	
	private TextArea traceOutput = null;

	public Server() 
	{
		JFrame.setDefaultLookAndFeelDecorated(true);
		this.window = new JFrame("Snaptaxi server");
		this.window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.window.addWindowListener(this);

		// create table
		{
			// Set screen layout style.
			Container container = window.getContentPane();
			container.setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.fill = GridBagConstraints.HORIZONTAL;
			c.weightx = 0.5;
			c.gridx = 0;
			c.gridy = 0;

			// Set Server infomation.
			{
				// IP and listen port.
				c.gridx = 0;
				c.gridy++;
				String serverInfo = "Server Listen Address - ";
				serverInfo += Snaptaxi.getSnaptaxi().getMyIP().getHostAddress();
				serverInfo += " : ";
				serverInfo += Integer.toString(Snaptaxi.getSnaptaxi().getListenPort());
				container.add(new JLabel(serverInfo), c);
			}

			{
				TextArea trace = new TextArea(5, 30);
				JScrollPane scrollPane = new JScrollPane(trace);
				scrollPane.setPreferredSize(new Dimension(450, 110));
				
				c.insets.top = 10;
				c.gridx = 0;
				c.gridy++;
				container.add(new JLabel("Trace Window"), c);

				c.insets.top = 0;
				c.gridx = 0;
				c.gridy++;
				container.add(scrollPane, c);
				
				this.traceOutput = trace;
			}

		}

		// pit window size.
		this.window.pack();
		
		// show window.
		this.window.setVisible(true);		
	}
	
	public void start() 
	{
		Snaptaxi.getSnaptaxi().startup(this);
	}

	private int count = 0;
	/**
	 * @param arg0
	 */
	public void windowActivated(WindowEvent arg0) 
	{
	}
	
	/**
	 * @param arg0
	 */
	public void windowClosed(WindowEvent arg0) 
	{
		
	}
	
	/**
	 * @param arg0
	 */
	public void windowClosing(WindowEvent arg0) 
	{
        Snaptaxi.getSnaptaxi().shutdown();		
	}
	
	/**
	 * @param arg0
	 */
	public void windowDeactivated(WindowEvent arg0) 
	{
		
	}
	
	/**
	 * @param arg0
	 */
	public void windowDeiconified(WindowEvent arg0) 
	{
		
	}
	
	/**
	 * @param arg0
	 */
	public void windowIconified(WindowEvent arg0) 
	{
		
	}
	
	/**
	 * @param arg0
	 */
	public void windowOpened(WindowEvent arg0) 
	{
		
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) 
	{
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                Server server = new Server();
                server.start();
            }
        });		
	}

	/* (non-Javadoc)
	 * @see snaptaxi.ServerListener#onLogin(snaptaxi.Address, snaptaxi.LoginMsg)
	 */
	public void onLogin(Address sender, LoginMsg msg) {
		printTrace(
				"onLogin",
				sender, 
				"ID/PASS: " + msg.getID() + "/" + msg.getPass()
				);
	}

	/* (non-Javadoc)
	 * @see snaptaxi.ServerListener#onLogout(snaptaxi.Address, snaptaxi.LogoutMsg)
	 */
	public void onLogout(Address sender, LogoutMsg msg) {
		printTrace(
				"onLogout",
				sender, 
				"ID: " + msg.getID()
				);
	}

	/* (non-Javadoc)
	 * @see snaptaxi.ServerListener#onRegist(snaptaxi.Address, snaptaxi.RegistMsg)
	 */
	public void onRegist(Address sender, RegistMsg msg) {
		printTrace(
				"onRegist",
				sender, 
				"ID/Pass[Type]: " + msg.getID() +
				"/" + msg.getPass() +
				"[" + msg.getType() + "]"
				);
	}

	/* (non-Javadoc)
	 * @see snaptaxi.ServerListener#onUnregist(snaptaxi.Address, snaptaxi.UnregistMsg)
	 */
	public void onUnregist(Address sender, UnregistMsg msg) {
		printTrace(
				"onUnregist",
				sender, 
				"ID: " + msg.getID()
				);
	}

	/* (non-Javadoc)
	 * @see snaptaxi.ServerListener#onSelect(snaptaxi.Address, snaptaxi.SelectMsg)
	 */
	public void onSelect(Address sender, SelectMsg msg) {
		printTrace(
				"onSelect",
				sender, 
				"Taxi/Passenger: " + msg.getTaxiID() + "/" + msg.getPassengerID()
				);
	}

	/* (non-Javadoc)
	 * @see snaptaxi.ServerListener#onUnselect(snaptaxi.Address, snaptaxi.UnselectMsg)
	 */
	public void onUnselect(Address sender, UnselectMsg msg) {
		printTrace(
				"onUnselect",
				sender, 
				"Taxi/Passenger: " + msg.getTaxiID() + "/" + msg.getPassengerID()
				);
	}

	/* (non-Javadoc)
	 * @see snaptaxi.ServerListener#onCall(snaptaxi.Address, snaptaxi.CallMsg)
	 */
	public void onCall(Address sender, CallMsg msg) {
		printTrace(
				"onCall",
				sender, 
				"Passenger: " + msg.getPassengerID()
				);
	}

	/* (non-Javadoc)
	 * @see snaptaxi.ServerListener#onCancelCall(snaptaxi.Address, snaptaxi.CancelCallMsg)
	 */
	public void onCancelCall(Address sender, CancelCallMsg msg) {
		printTrace(
				"onCancelCall",
				sender, 
				"Passenger: " + msg.getPassengerID()
				);
	}

	/* (non-Javadoc)
	 * @see snaptaxi.ServerListener#onChangeState(snaptaxi.Address, snaptaxi.ChangeStateMsg)
	 */
	public void onChangeState(Address sender, ChangeStateMsg msg) {
		printTrace(
				"onChangeState",
				sender, 
				"Taxi[NewState]: " + msg.getTaxiID() + "[" + msg.getState() + "]"
				);
	}

	/* (non-Javadoc)
	 * @see snaptaxi.ServerListener#onGPSPosition(snaptaxi.Address, snaptaxi.GPSPositionMsg)
	 */
	public void onGPSPosition(Address sender, GPSPositionMsg msg) {
		printTrace(
				"onGPSPosition",
				sender,
				"ID[X,Y]: " + msg.getID() + "[" + msg.getPosX() + "," + msg.getPosY() + "]"
				);
	}

	/* (non-Javadoc)
	 * @see snaptaxi.ServerListener#onDisconnect(snaptaxi.Address, snaptaxi.DisconnectMsg)
	 */
	public void onDisconnect(Address sender, DisconnectMsg msg) {
		printTrace(
				"onDisconnect",
				sender,
				"ID: " + msg.getID()
				);
	}
	
	private void printTrace(String tName ,Address sender, String msg)
	{
		this.traceOutput.insert(
				"From:" + sender.getIP().toString()+"["+sender.getPort()+"]"+
				" (" + tName + ") " +
				msg,
				0
				);
	}
}