
package snaptaxi;

import java.util.*;

/**
 * Store & manage customer call.
 * All taxi call state are here.
 * 
 * @author Korea Univ. SE Team #1
 * @version 1.0
 * @since 2004.11.14
 * [2004.11.14.] ~ Add source.
 * [2004.11.14.ȫ/] ~ Implement source
 * [2004.11.19.] ~ Update document.
 * [2004.11.19.] ~ Update document.
 * [2004.11.20.] ~ reimplement and update document.
 * [2004.11.21.] - end intial code.
 */
public class CallMgr 
{
	// taxi search range. (unit: meter)
	final private int SEARCH_RADIUS = 10;

	// Store call state here. <PassengerID, CustomerState> format.
	private HashMap callStateMap;

	// taxi list. from MessageListner.
	private CustomerMgr taxiMgr;
	
	// passenger list. from MessageListner.
	private CustomerMgr passengerMgr;
	
	
	/**
	 * Constructor.
	 * Initialize all property.
	 * Warning! We provide default taxiMgr & passengerMgr.
	 * These managers should be from MessageListener
	 * But, to avoid runtime crash we give default manager.
	 */
	public CallMgr() 
	{
		callStateMap = new HashMap();
		taxiMgr = new CustomerMgr();
		passengerMgr = new CustomerMgr();
	}
	
	/**
	 * Change taxi list.
	 * 
	 * @param taxiMgr - taxi list for replace.
	 */
	public void setTaxiMgr(CustomerMgr taxiMgr) 
	{
		this.taxiMgr = taxiMgr;
	}
	
	/**
	 * Change passenger list
	 * 
	 * @param passengerMgr - passenger list for replace.
	 */
	public void setPassengerMgr(CustomerMgr passengerMgr) 
	{
		this.passengerMgr = passengerMgr;
	}
	
	/**
	 * Add call request.
	 * 
	 * @param callMsg - call request for inserting.
	 */
	public void call(CallMsg callMsg) 
	{
		CallState matchState = new CallState();
		{
			// List of callable taxi -> matchState
			// Find taxi with distance (between passenger) - Critical source
			// When there's no matching taxi, extand radius and searching again.
			matchState.setTaxiList(
					taxiMgr.find(
							callMsg.getPosX(),
							callMsg.getPosY(), SEARCH_RADIUS)
							);

			matchState.setState(CallState.STATE_REQUEST);
		}

		// Save call information
		// Save into HashMap(callStateMap) id and callState
		callStateMap.put( callMsg.getPassengerID(), matchState );

		// send CALL messege to each taxi
		{
			Iterator iter = matchState.getTaxiList().iterator();
			while(iter.hasNext()) {
				CustomerState cs = (CustomerState) iter.next();
				Snaptaxi.getSnaptaxi().sendMessage( cs.getAddress(), callMsg );
			}
		}
	}

	/**
	 * Remove call request.
	 * 
	 * @param cancelCallMsg - call request for remove.
	 */
	public void cancelCall(CancelCallMsg cancelCallMsg) 
	{
		if (!this.callStateMap.containsKey(cancelCallMsg.getPassengerID()))
		{
			// if cancel which is not issued. ignore request.
			return ;
		}

		// Searching callStateMap(HashMap) with passengerID from CancleCallMsg		
		// SEND cancle call msg to all taxi which are matched by searching from call information
		{
			CallState tempCallState = 
				(CallState) callStateMap.get( cancelCallMsg.getPassengerID() );
			Iterator iter = tempCallState.getTaxiList().iterator();
			while(iter.hasNext()) {
				CustomerState cs = (CustomerState) iter.next();
				Snaptaxi.getSnaptaxi().sendMessage(cs.getAddress(), cancelCallMsg);
			}

			// Delete callState from hashtable that matched with id of cancelCallMsg.
			callStateMap.remove(cancelCallMsg.getPassengerID());
		}
	}
	
	/**
	 * Update call state by selection.
	 * 
	 * @param selectMsg - selection data.
	 */
	public void select(SelectMsg selectMsg) 
	{
		// Get request.
		CallState request = (CallState) this.callStateMap.get(selectMsg.getPassengerID());

		// Get Caller.
		CustomerState caller = this.taxiMgr.getState(selectMsg.getTaxiID());

		// if cannot find taxi call request.
		if (null == request) {
			// send aleady select message to taxi (who is select passenger).
			Snaptaxi.getSnaptaxi().sendMessage(
					caller.getAddress(),
					new SelectResultMsg(
							SelectResultMsg.RESULT_SELECT_FAIL,
							selectMsg.getPassengerID()
							)
					);
			return ;
		}

		// Broadcast select message to all taxi.
		Iterator iter = request.getTaxiList().iterator();
		while (iter.hasNext()) {
			CustomerState taxi = (CustomerState) iter.next();
			if (taxi == caller) {
				Snaptaxi.getSnaptaxi().sendMessage(
						taxi.getAddress(),
						new SelectResultMsg(
								SelectResultMsg.RESULT_SELECT_SUCCESS,
								selectMsg.getPassengerID()
								)
						);
			} else {
				Snaptaxi.getSnaptaxi().sendMessage(
						taxi.getAddress(),
						new SelectResultMsg(
								SelectResultMsg.RESULT_SELECTED,
								selectMsg.getPassengerID()
								)
						);
			}
		}

		// change caller taxi state to busy.
		caller.setState(CustomerState.STATE_BUSY);
		
		// change call state to answer.
		request.setState(CallState.STATE_ANSWER);
		
		// send select result message to passenger.
		{
			CustomerState passengerState = 
				this.passengerMgr.getState(selectMsg.getPassengerID());
			if (null != passengerState) {
				Snaptaxi.getSnaptaxi().sendMessage(
						passengerState.getAddress(), 
						new SelectResultMsg(
								SelectResultMsg.RESULT_SELECT_SUCCESS,
								selectMsg.getTaxiID()
								)
						);
			}
		}
	}
	
	/**
	 * Update call state by unselection.
	 * 
	 * @param unselectMsg - unselection data.
	 */
	public void unselect(UnselectMsg unselectMsg)
	{
		// Get request.
		CallState request = (CallState) this.callStateMap.get(unselectMsg.getPassengerID());

		// Get Caller.
		CustomerState caller = this.taxiMgr.getState(unselectMsg.getTaxiID());
		
		// Send unselect message.
		Iterator iter = request.getTaxiList().iterator();
		while(iter.hasNext()) {
			CustomerState cs = (CustomerState) iter.next();
			Snaptaxi.getSnaptaxi().sendMessage(
					cs.getAddress(),
					new SelectResultMsg(
							SelectResultMsg.RESULT_UNSELECT,
							unselectMsg.getPassengerID()
							)
					);
		}

		// change caller state to ready.
		caller.setState(CustomerState.STATE_READY);
		
		// change call satte to answer.
		request.setState(CallState.STATE_REQUEST);

		// send select result message to passenger.
		{
			CustomerState passengerState = 
				this.passengerMgr.getState(unselectMsg.getPassengerID());
			if (null != passengerState) {
				Snaptaxi.getSnaptaxi().sendMessage(
						passengerState.getAddress(), 
						new SelectResultMsg(
								SelectResultMsg.RESULT_UNSELECT,
								unselectMsg.getTaxiID()
								)
						);
			}
		}
	}
}