
package snaptaxi;


/**
 * @author Korea Univ. SE Team #1
 * @version 1.0
 * @since 2004.11.14
 * [2004.11.14.] ~ Add source.
 * [2004.11.15.] ~ Add msg inheritance.
 * [2004.11.15.] ~ Message inheritance addition.
 * [2004.11.21.] - Add state constant.
 */
public class ChangeStateMsg extends Message 
{
	/**
	 * Customer State code.
	 * STATE_ERROR - initial state.
	 * STATE_READY - ready for call.
	 * STATE_BUSY - on busy. (waiting result. or not accept call state)
	 */
	private int state;

	// initial state
	final public static int STATE_ERROR = 0;

	// ready for call.
	final public static int STATE_READY = 1;

	// on busy. (waiting result. or not accept call state)
	final public static int STATE_BUSY = 2;

	// ID of taxi who wants change state.
	private String taxiID;

	/**
	 * Default constructor.
	 * It means null msg.
	 */
	public ChangeStateMsg() 
	{
		state = STATE_ERROR;
		taxiID = "";		
	}

	/**
	 * Set state for change.
	 * Set once only.
	 * @param taxiID ~ taxi ID
	 * @param state  ~ state
	 */
	public ChangeStateMsg(String taxiID, int state) 
	{
		this.state = state;
		this.taxiID = taxiID;		
	}
	
	/**
	 * Check ID of Taxi who wants change state.
	 * @return String ~ taxi ID
	 */
	public String getTaxiID() 
	{
		return taxiID;		
	}
	
	/**
	 * @return int ~ taxi state
	 * STATE_ERROR - initial state.
	 * STATE_READY - ready for call.
	 * STATE_BUSY - on busy. (waiting result. or not accept call state)
	 */
	public int getState() 
	{
		return state;		
	}
}