
package snaptaxi;


/**
 * @author Korea Univ. SE Team #1
 * @version 1.0
 * @since 2004.11.14
 * [2004.11.14.] ~ Add source.
 * [2004.11.14.ȫ/] ~ implemantation
 * [2004.11.20.] - Add state constant.
 * 						STATE_ERROR - initial state.
 * 						STATE_READY - ready for call.
 * 						STATE_BUSY - on busy. (waiting result. or not accept call state)
 */
public class CustomerState 
{
	/**
	 * STATE_ERROR - initial state.
	 * STATE_READY - ready for call.
	 * STATE_BUSY - on busy. (waiting result. or not accept call state)
	 */
	private int state;

	// initial state
	final public static int STATE_ERROR = 0;
	
	// ready for call.
	final public static int STATE_READY = 1;
	
	// on busy. (waiting result. or not accept call state)
	final public static int STATE_BUSY = 2;

	// position x. (unit: meter)
	private int posX;
	
	// position y. (unit: meter)
	private int posY;
	
	// internet address.
	private Address address;

	/**
	 * Default constructor. Initialize.
	 *
	 */
	public CustomerState() 
	{
		posX = 0;
		posY = 0;
		state = STATE_ERROR;
		address = new Address();
	}
	
	/**
	 * Change postion.
	 * @param posX - x position. (unit : meter)
	 * @param posY - y position. (unit : meter)
	 */
	public void setPos(int posX, int posY) 
	{
		this.posX = posX;
		this.posY = posY;
	}
	
	/**
	 * Check x position.
	 * @return int - position. (unit : meter)
	 */
	public int getPosX() 
	{
		return posX;
	}
	
	/**
	 * Check y position.
	 * @return int - position. (unit : meter)
	 */
	public int getPosY() 
	{
		return posY;
	}
	
	/**
	 * Change current state.
	 * @param state
	 * STATE_ERROR - initial state.
	 * STATE_READY - ready for call.
	 * STATE_BUSY - on busy. (waiting result. or not accept call state)
	 */
	public void setState(int state) 
	{
		this.state = state;
	}
	
	/**
	 * Check current state.
	 * @return int
	 * STATE_ERROR - initial state.
	 * STATE_READY - ready for call.
	 * STATE_BUSY - on busy. (waiting result. or not accept call state)
	 */
	public int getState() 
	{
		return state;
	}
	
	/**
	 * Change customer internet address.
	 * @param address - internet address.
	 */
	public void setAddress(Address address) 
	{
		this.address = address;
	}
	
	/**
	 * Check customer internet address.
	 * @return snaptaxi.Address
	 */
	public Address getAddress() 
	{
		return address;
	}
}