
package snaptaxi;


/**
 * @author Korea Univ. SE Team #1
 * @version 1.0
 * @since 2004.11.14
 * [2004.11.14.] ~ Add source.
 * [2004.11.15.] ~ Initial coding complete
 * [2004.11.15.] ~ Add msg inheritance.
 * [2004.11.20.] ~ Add state constant.
 * 						Login result state flag
 *						LOGIN_ERROR				: error
 *						LOGIN_SUCCESS			: login succession
 *						LOGIN_UNREGISTER		: unresiter customer
 *						LOGIN_INVALID_PASS		: invalid password
 */
public class LoginResultMsg extends Message 
{
	
	/**
	 * Login result state flag
	 * LOGIN_ERROR				: error
	 * LOGIN_SUCCESS			: login succession
	 * LOGIN_UNREGISTER			: unresiter customer
	 * LOGIN_INVALID_PASS		: invalid password
	 */
	private int loginCode;
	
	// error
	final static public int LOGIN_ERROR = 0;
	
	// login succession
	final static public int LOGIN_SUCCESS = 1;
	
	// unregister customer
	final static public int LOGIN_UNREGISTER = 2;
	
	// invalid password.
	final static public int LOGIN_INVALID_PASS = 3;
	
	/**
	 * @param loginCode
	 * LOGIN_ERROR				: error
	 * LOGIN_SUCCESS			: login succession
	 * LOGIN_UNREGISTER			: unresiter customer
	 * LOGIN_INVALID_PASS		: invalid password
	 */
	public LoginResultMsg(int loginCode) 
	{
		this.loginCode = loginCode;		
	}
	
	public LoginResultMsg() 
	{
		this.loginCode = LOGIN_ERROR;
	}
	
	/**
	 * Check login is success.
	 * @return this.loginCode == LOGIN_SUCCESS
	 */
	public boolean isSuccess() 
	{
		return (this.loginCode == LOGIN_SUCCESS);
	}
	
	/**
	 * Check login is fail. Caused by unregistered customer.
	 * @return this.loginCode == LOGIN_UNREGISTER
	 */
	public boolean isNotRegist() 
	{
		return (this.loginCode == LOGIN_UNREGISTER);		
	}
	
	/**
	 * Check login is fail. Caused by invalid password.
	 * @return this.loginCode == LOGIN_INVALID_PASS
	 */
	public boolean isInvalidPass() 
	{
		return (this.loginCode == LOGIN_INVALID_PASS);
	}
}