
package snaptaxi;

import java.util.HashMap;

/**
 * Handle all message.
 * Actually this is main routine.
 * 
 * @author Korea Univ. SE Team #1
 * @version 1.0
 * @since 2004.11.14
 * [2004.11.14.] ~ Add source.
 * [2004.11.19.] - implement onLogin()
 * [2004.11.21.] - End initial code.
 * [2004.11.21.] - Update Document.
 */
public class MessageListener 
{
	// Passenger list.
	private CustomerMgr passengerMgr;

	// Taxi list.
	private CustomerMgr taxiMgr;

	// Call Request list.
	private CallMgr callMgr;

	// Data Base.
	private DB db;

	// Address -> ID Map for find ID from IPAddress.
	private HashMap addressToIDMap;

	// initialize listener.
	public MessageListener() 
	{
	    this.db = new DB();

	    this.callMgr = new CallMgr();

	    this.passengerMgr = new CustomerMgr();
		this.taxiMgr = new CustomerMgr();

		this.callMgr.setPassengerMgr(this.passengerMgr);
		this.callMgr.setTaxiMgr(this.taxiMgr);

		this.addressToIDMap = new HashMap();
	}

	/**
	 * Change database.
	 * @param db - new database instance.
	 */
	public void setDB(DB db) 
	{
		this.db = db;
	}

	/**
	 * handle login message.
	 * @param sender - client address who send message.
	 * @param msg - data which is sent by client.
	 */
	public void onLogin(Address sender, LoginMsg msg) 
	{
        if (this.serverListener != null) {
			this.serverListener.onLogin(sender, msg);
		}
	    CustomerData cd = this.db.findCustomerData(msg.getID());
	    if (null == cd) 
	    {
			// send not registered customer.
	    	Snaptaxi.getSnaptaxi().sendMessage(
	    			sender, new LoginResultMsg(LoginResultMsg.LOGIN_UNREGISTER));
			return ;
	    }
	    
	    if ( msg.getPass().compareTo(cd.getPass()) == 0 ) 
	    {
	    	// invalid password.
	    	Snaptaxi.getSnaptaxi().sendMessage(
	    			sender, new LoginResultMsg(LoginResultMsg.LOGIN_INVALID_PASS));
	    	return ;
	    }
	    
	    if ( this.passengerMgr.isLogin(msg.getID()) ||
	    	 this.taxiMgr.isLogin(msg.getID())) 
	    {
	    	// If aleady login.
	    	LogoutMsg logoutMsg = new LogoutMsg(msg.getID());
	    	
	    	// logout now.
	    	this.onLogout(sender, logoutMsg);
	    }
	    
	    // regist customer.
	    switch (cd.getType()) {
	    case CustomerData.CUSTOMER_ERROR: // error.
		default: // unknown type.
			Snaptaxi.getSnaptaxi().sendMessage(
					sender, new LoginResultMsg(LoginResultMsg.LOGIN_ERROR));
			return ;
    	case CustomerData.CUSTOMER_PASSENGER: // passenger
    		// regist passenger.
    		this.passengerMgr.login(msg.getID());
    		break;
   		case CustomerData.CUSTOMER_TAXI: // taxi
   			// regist taxi.
   			this.taxiMgr.login(msg.getID());
   			break;
	    }

	    // regist link.
	    this.addressToIDMap.put(sender, msg.getID());
	    
	    // send login success.
    	Snaptaxi.getSnaptaxi().sendMessage(
    			sender, new LoginResultMsg(LoginResultMsg.LOGIN_SUCCESS));
	}

	/**
	 * handle logout message.
	 * @param sender - client address who send message.
	 * @param msg - data which is sent by client.
	 */
	public void onLogout(Address sender, LogoutMsg msg) 
	{
		if (this.serverListener != null) {
			this.serverListener.onLogout(sender, msg);
		}
		CustomerData cd = this.db.findCustomerData(msg.getID());
		if (null == cd) {
			return ;
		}
		switch(cd.getType())
		{
		case CustomerData.CUSTOMER_ERROR:
		default:
			// Error.
			return;
		case CustomerData.CUSTOMER_PASSENGER:
			// Passenger
			this.passengerMgr.logout(msg.getID());
			break;
		case CustomerData.CUSTOMER_TAXI:
			// Taxi
			this.taxiMgr.logout(msg.getID());
			break;
		}
	}

	/**
	 * Handle regist message.
	 * @param sender - client address who send message.
	 * @param msg - data which is sent by client.
	 */
	public void onRegist(Address sender, RegistMsg msg) 
	{
		if (this.serverListener != null) {
			this.serverListener.onRegist(sender, msg);
		}
		if (this.db.isExist(msg.getID())) {
			Snaptaxi.getSnaptaxi().sendMessage(
					sender, new RegistResultMsg(RegistResultMsg.REGIST_REGISTERED)
					);
			
			return ;
		}
		if (this.db.regist(msg)) {
			Snaptaxi.getSnaptaxi().sendMessage(
					sender,
					new RegistResultMsg(RegistResultMsg.REGIST_SUCCESS)
					);
		} else {
			Snaptaxi.getSnaptaxi().sendMessage(
					sender,
					new RegistResultMsg(RegistResultMsg.REGIST_DBERROR)
					);
		}
	}

	/**
	 * Handle unregist message.
	 * @param sender - client address who send message.
	 * @param msg - data which is sent by client.
	 */
	public void onUnregist(Address sender, UnregistMsg msg) 
	{
		if (this.serverListener != null) {
			this.serverListener.onUnregist(sender, msg);
		}
		this.db.unregist(msg);
	}
	
	/**
	 * Handle taxi select message.
	 * @param sender - client address who send message.
	 * @param msg - data which is sent by client.
	 */
	public void onSelect(Address sender, SelectMsg msg) 
	{
		if (this.serverListener != null) {
			this.serverListener.onSelect(sender, msg);
		}
		this.callMgr.select(msg);
	}
	
	/**
	 * 
	 * @param sender - client address who send message.
	 * @param msg - data which is sent by client.
	 */
	public void onUnselect(Address sender, UnselectMsg msg) 
	{
		if (this.serverListener != null) {
			this.serverListener.onUnselect(sender, msg);
		}
		this.callMgr.unselect(msg);
	}
	
	/**
	 * @param sender - client address who send message.
	 * @param msg - data which is sent by client.
	 */
	public void onCall(Address sender, CallMsg msg) 
	{
		if (this.serverListener != null) {
			this.serverListener.onCall(sender, msg);
		}
		this.callMgr.call(msg);
	}
	
	/**
	 * @param sender - client address who send message.
	 * @param msg - data which is sent by client.
	 */
	public void onCancelCall(Address sender, CancelCallMsg msg) 
	{
		if (this.serverListener != null) {
			this.serverListener.onCancelCall(sender, msg);
		}
		this.callMgr.cancelCall(msg);
	}

	/**
	 * Get CustomerState by id.
	 * If cannot find. result is null.
	 * Ignore it's type.
	 * @param id - id name for find customer state.
	 * @return CustomerState - result state.
	 */
	private CustomerState getCustomerState(String id)
	{
		CustomerState cs = this.passengerMgr.getState(id);
		if (null == cs) {
			cs = this.taxiMgr.getState(id);
		}
		return cs;
	}
	
	/**
	 * @param sender - client address who send message.
	 * @param msg - data which is sent by client.
	 */
	public void onChangeState(Address sender, ChangeStateMsg msg) 
	{
		if (this.serverListener != null) {
			this.serverListener.onChangeState(sender, msg);
		}
		CustomerState cs = this.taxiMgr.getState(msg.getTaxiID());
		if (null != cs) {
			cs.setState(msg.getState());
		}
	}
	
	/**
	 * @param sender - client address who send message.
	 * @param msg - data which is sent by client.
	 */
	public void onGPSPosition(Address sender, GPSPositionMsg msg) 
	{
		if (this.serverListener != null) {
			this.serverListener.onGPSPosition(sender, msg);
		}
		CustomerState cs = this.getCustomerState(msg.getID());
		if (null != cs) {
			cs.setPos(msg.getPosX(), msg.getPosY());
		}
	}
	
	/**
	 * @param sender - client address who send message.
	 * @param msg - data which is sent by client.
	 */
	public void onDisconnect(Address sender, DisconnectMsg msg) 
	{
		if (this.serverListener != null) {
			this.serverListener.onDisconnect(sender, msg);
		}
		CustomerData cd = this.db.findCustomerData(msg.getID());
		if (null == cd) {
			return ;
		}
		switch(cd.getType())
		{
		case CustomerData.CUSTOMER_ERROR:
		default:
			// Error.
			return;
		case CustomerData.CUSTOMER_PASSENGER:
			// Passenger
			this.passengerMgr.logout(msg.getID());
			break;
		case CustomerData.CUSTOMER_TAXI:
			// Taxi
			this.taxiMgr.logout(msg.getID());
			break;
		}
	}
	
	private ServerListener serverListener = null;

	public void setServerListener(ServerListener serverListener)
	{
		this.serverListener = serverListener;
	}
}