
package snaptaxi;

import java.net.*;
import java.io.*;

/**
 * This is server network module.
 * Receive packet from client and convert packet to message.
 * If message is created, redirect message to listner.
 * This send message also. but, only snaptaxi class use this method.
 * 
 * @author Korea Univ. SE Team #1
 * @version 1.0
 * @since 2004.11.14
 * [2004.11.14.] ~ Add source.
 * [2004.11.14.] ~ initial code.
 * [2004.11.14.] ~ need standard communication method server and client.
 * [2004.11.19.] ~ add sendMessage() method.
 * [2004.11.19.] ~ change listener style.
 * 						From multi-listener to single-listener.
 * [2004.11.19.] ~ Implementation Complete.
 * [2004.11.20.] ~ Roll back.
 * [2004.11.21.] - Add getIP(), getPort() method and fill dummy code. (need implement)
 */
public class NetServer extends Thread 
{
	private MessageListener listener;

	private DatagramSocket socket=null;
	private DatagramPacket packetOut=null, packetIn=null;
	private byte inputB[] = new byte[1024];
	private int port = 3000;

	public NetServer()
	{
	    this.listener = new MessageListener();
	    
	    // Initialize sockect and open.
	       try {

	          socket = new DatagramSocket(port); 
	       }
	       catch(IOException e){

		          System.out.println("Failure: "+e);

		   }
	}
	
	/**
	 * Wait & Process Message from Network.
	 */
	public void run()
	{
	    // Make ObjectOutputStream & ByteArrayOutputStream.
		// also, Receive Packet from network.
		// make messages to Object stream.
		// & get class name from Object stream.
		for(;;) {
			try {
				int len = inputB.length;
				packetIn = new DatagramPacket(inputB, len);
		        socket.receive(packetIn);
		        //Receive Packet from network.
		        
		        ByteArrayInputStream inputBuf = new ByteArrayInputStream(inputB);
		        ObjectInputStream inputObj = new ObjectInputStream(inputBuf);
		        //Make ObjectOutputStream & ByteArrayOutputStream.
		        
		        Object packet = inputObj.readObject();
		        
		        if(packet.getClass().getName() == "snaptaxi.RegistMsg") {
		        	// if Class name is RegistMsg, Regist Object to the listener.
		        	this.listener.onRegist(
		        			new Address(packetIn.getAddress(), packetIn.getPort()),
							(RegistMsg) packet);
		        }
		        else if(packet.getClass().getName() == "snaptaxi.LoginMsg") {
		        	// if Class name is LoginMsg, Regist Object to the listener.
			        this.listener.onLogin(
		        			new Address(packetIn.getAddress(), packetIn.getPort()),
		        			(LoginMsg) packet);
		        }
		        else if(packet.getClass().getName() == "snaptaxi.UnregistMsg") {
		        	// if Class name is UnregistMsg, Regist Object to the listener.
		          	this.listener.onUnregist(
		        			new Address(packetIn.getAddress(), packetIn.getPort()),
		        			(UnregistMsg) packet);
		        }
		        else if(packet.getClass().getName() == "snaptaxi.LogoutMsg") {
		        	// if Class name is LogoutMsg, Regist Object to the listener.
		          	this.listener.onLogout(
		        			new Address(packetIn.getAddress(), packetIn.getPort()),
		        			(LogoutMsg) packet);
		         }
		        else if(packet.getClass().getName() == "snaptaxi.DisconnectMsg") {
		        	// if Class name is DisconnectMsg, Regist Object to the listener.
		          	this.listener.onDisconnect(
		        			new Address(packetIn.getAddress(), packetIn.getPort()),
		        			(DisconnectMsg) packet);
		        }
		        else if(packet.getClass().getName() == "snaptaxi.CallMsg") {
		        	// if Class name is CallMsg, Regist Object to the listener.
		          	this.listener.onCall(
		        			new Address(packetIn.getAddress(), packetIn.getPort()),
		        			(CallMsg) packet);
		        }
		        else if(packet.getClass().getName() == "snaptaxi.SelectMsg") {
		        	// if Class name is SelectMsg, Regist Object to the listener.
		          	this.listener.onSelect(
		        			new Address(packetIn.getAddress(), packetIn.getPort()),
		        			(SelectMsg) packet);
		        	SelectMsg temp = (SelectMsg) inputObj.readObject();
		        }
		        else if(packet.getClass().getName() == "snaptaxi.CancelCallMsg") {
		        	// if Class name is CancelCallMsg, Regist Object to the listener.
		          	this.listener.onCancelCall(
		        			new Address(packetIn.getAddress(), packetIn.getPort()),
		        			(CancelCallMsg) packet);
		        }
		        else if(packet.getClass().getName() == "snaptaxi.UnselectMsg") {
		        	// if Class name is UnselectMsg, Regist Object to the listener.
		          	this.listener.onUnselect(
		        			new Address(packetIn.getAddress(), packetIn.getPort()),
		        			(UnselectMsg) packet);
		        }
		        else if(packet.getClass().getName() == "snaptaxi.ChangeStateMsg") {
		        	// if Class name is ChangeStateMsg, Regist Object to the listener.
		          	this.listener.onChangeState(
		        			new Address(packetIn.getAddress(), packetIn.getPort()),
		        			(ChangeStateMsg) packet);
		        }
		        else if(packet.getClass().getName() == "snaptaxi.GPSPositionMsg") {
		        	// if Class name is GPSPositionMsg, Regist Object to the listener.
		          	this.listener.onGPSPosition(
		        			new Address(packetIn.getAddress(), packetIn.getPort()),
		        			(GPSPositionMsg) packet);
		        }
			} catch (IOException i){ // do nothing if it has the IOException -_-
				i.printStackTrace();
			} catch (ClassNotFoundException c){// do nothing if it has ClassNotFound ALSO -_-v
											   // cause it's BULLSHIT LIKE YOU -_-b
				c.printStackTrace();
			}
		}
	}
	
	/**
	 * Regist(or set) Network Listener.
	 * @param msgListener ~ Message Listener to regist.
	 */
	public void setListener(MessageListener msgListener) 
	{
		this.listener = msgListener;
	}

	/**
	 * Send message to client.
	 * @param address ~ client internet address.
	 * @param msg ~ data for send.
	 */
	public void sendMessage(Address address, Message msg)
	{
		try {
			ByteArrayOutputStream outputBuf = new ByteArrayOutputStream();
	        ObjectOutputStream outputObj = new ObjectOutputStream(outputBuf);   
	        //Create ByteArrayOutputStream & ObjectOutputStream.
	        
	        outputObj.writeObject(msg);
	        //Write Message Object to ObjectOutputStream.
	        //Actually, those Object will be written in outputbuf.
	        	        
	        packetOut = new DatagramPacket(
	        		outputBuf.toByteArray(), outputBuf.size(),
					address.getIP(), address.getPort());
	        //Create Packet from information of ObjectOutputStream & Address Object.
	        socket.send(packetOut); 
	        outputObj.close();
	        outputBuf.close();
	        //Close It.
		} catch(IOException i){// do nothing if it has IOException -_-v
			                   // cause it's BULLSHIT LIKE YOU -_-b
			
		}
	}

	/**
	 *  IPּҸ ȮѴ.
	 * @return InetAddress -  IPּ.
	 */
	public InetAddress getIP()
	{
		InetAddress addr = null;
		try {
			addr = InetAddress.getByName("127.0.0.1");
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
		return addr;
	}

	/**
	 *   Listen ϰ ִ ּҸ ´.
	 * @return int - port ȣ.
	 */
	public int getPort()
	{
		return this.port;
	}
}