
package snaptaxi;

/**
 * @author Korea Univ. SE Team #1
 * @version 1.0
 * @since 2004.11.14
 * [2004.11.14.] ~ Add source.
 * [2004.11.15.] ~ Add msg inheritance.
 * [2004.11.20.] - Add constant
 * 						RESULT_ERROR : error
 * 						RESULT_SELECT : select success
 * 						RESULT_ALREADY_SELECT : passenger already selected
 * [2004.11.20.] - extends from Message.
 * [2004.11.20.] - remove setState() method.
 * [2004.11.20.] - Add passenger id.
 * [2004.11.20.] - Add constant
 * 						RESULT_UNSELECT : unselect passenger.
 * [2004.11.20.] - Change constant name. from RESULT_SUCCESS to RESULT_UNSELECT
 * [2004.11.20.] - Change method name.
 * [2004.11.20.] - Change constant name. Add constant.
 * 						RESULT_ERROR : error
 * 						RESULT_SELECT_SUCCESS : select success
 * 						RESULT_SELECT_FAIL : select fail.
 * 						RESULT_SELECTED : passenger selected by other taxi.
 * 						RESULT_UNSELECT : passenger is unselected.
 * [2004.11.21.] - Change passengerID to selecterID.
 * [2004.11.29.] - Add isSuccess() isFail() method.
 */
public class SelectResultMsg extends Message
{
	/**
	 * Select passenger flag.
	 * RESULT_ERROR : error
	 * RESULT_SELECT_SUCCESS : select success
	 * RESULT_SELECT_FAIL : select fail.
	 * RESULT_SELECTED : passenger selected by other taxi.
	 * RESULT_UNSELECT : passenger is unselected.
	 */
	private int resultCode;
	
	private String selecterID;

	// error
	final static public int RESULT_ERROR = 0;
	
	// select success
	final static public int RESULT_SELECT_SUCCESS = 1;

	// passenger already selected.
	final static public int RESULT_SELECT_FAIL = 2;

	// passenger selected.
	final static public int RESULT_SELECTED = 3;
	
	// passenger unselected.
	final static public int RESULT_UNSELECT = 4;
	
	/**
	 * @param selectCode
	 * RESULT_ERROR : error
	 * RESULT_SELECT : select success
	 * RESULT_ALREADY_SELECT : passenger already selected
	 * RESULT_UNSELECT : unselect passenger.
	 */
	public SelectResultMsg(int resultCode, String selecterID)
	{
		this.resultCode = resultCode;
		this.selecterID = selecterID;
	}

	/**
	 * Using default construct means null result.
	 */
	public SelectResultMsg() 
	{
		this.resultCode = RESULT_ERROR;
		this.selecterID = "";
	}
	
	
	/**
	 * Get result code.
	 * @return
	 * RESULT_ERROR : error
	 * RESULT_SELECT_SUCCESS : select success
	 * RESULT_SELECT_FAIL : select fail.
	 * RESULT_SELECTED : passenger selected by other taxi.
	 * RESULT_UNSELECT : passenger is unselected.
	 */
	public int getState()
	{
		return this.resultCode;
	}

	/**
	 * Get passenger ID who is select.
	 * @return passenger ID
	 */
	public String getSelecterID()
	{
		return this.selecterID;
	}
	/**
	 * Check login is success.
	 * @return this.loginCode == RESULT_SELECT_SUCCESS
	 */
	public boolean isSuccess() 
	{
		return (this.resultCode == RESULT_SELECT_SUCCESS);
	}
	/**
	 * Check login is success.
	 * @return this.loginCode == RESULT_SELECT_FAIL
	 */
	public boolean isFail() 
	{
		return (this.resultCode == RESULT_SELECT_FAIL);
	}
	
}