
package snaptaxi;

import java.net.InetAddress;

/**
 * Snaptaxi system interface.
 * This is singleton class.
 * and, you can access system by using getSnaptaxi() method.
 * 
 * @author Korea Univ. SE Team #1
 * @version 1.0
 * @since 2004.11.14
 * [2004.11.14.] ~ Add source.
 * [2004.11.19.] ~ Add sendMessage method.
 * [2004.11.21.] - Add getMyID(), getListenPort() method.
 * [2004.12.14.] - Add ServerListener.
 */
public class Snaptaxi 
{
	// snaptaix singleton instance.
	private static Snaptaxi singleton = null;

	// Network virtual device.
	private NetServer server;

	// system runing flag.
	boolean runFlag;

	// Server Event Listner.
	private ServerListener listener;

	/**
	 * Default constructor.
	 * 
	 * You cannot access this method.
	 * So, you cannot make instance by new operation.
	 * Use getSnaptaxi() static method for access.
	 * 
	 */
	private Snaptaxi() 
	{
	    this.server = new NetServer();
	    this.runFlag = false;
	    this.listener = null;
	}
	
	/**
	 * Singleton accesser.
	 * 
	 * @return snaptaxi.Snaptaxi.
	 */
	public static Snaptaxi getSnaptaxi() 
	{
	    if (null == Snaptaxi.singleton) {
	        Snaptaxi.singleton = new Snaptaxi();
	    }
		return Snaptaxi.singleton;
	}

	/**
	 * Start snaptaxi system.
	 * If aleady run. Do nothing.
	 */
	public void startup(ServerListener listener) 
	{
		if (this.runFlag) 
		{
			return ;
		}
		MessageListener msgListener = new MessageListener();
		msgListener.setServerListener(listener);
	    this.server.setListener(msgListener);
		this.server.start();
		
		this.runFlag = true;
	}

	/**
	 * Shutdown snaptaxi system.
	 * It takes time for thead-end.
	 */
	public void shutdown() 
	{
		this.runFlag = false;
		
		this.server = null;
		Snaptaxi.singleton = null;
		System.gc();
	}
	
	/**
	 * Send message to client.
	 */
	public void sendMessage(Address address, Message msg)
	{
		if (false == this.runFlag) {
			return ;
		}

		this.server.sendMessage(address, msg);
	}

	/**
	 *  IPּҸ ȮѴ.
	 * @return InetAddress -  IPּ.
	 */
	public InetAddress getMyIP()
	{
		return this.server.getIP();
	}

	/**
	 *   Listen ϰ ִ ּҸ ´.
	 * @return int - port ȣ.
	 */
	public int getListenPort()
	{
		return this.server.getPort();
	}
}