
package snaptaxi.passenger;

import snaptaxi.*;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Vector;

import snaptaxi.passenger.Login;


/**
 * @author Korea Univ. SE Team #1
 * @version 1.0
 * @since 2004.11.20
 * [2004.11.20./] ~ Add source.
 * [2004.11.20./] ~ Add getPassenger.
 * [2004.11.20./] ~ Add sendMessage.
 * [2004.11.29.] ~ Implement Networkpart in every single method.
 * [2004.11.29.] ~ onLoginResult deleted cause PassengerListener already has one. 
 * [2004.11.29./] ~ Add test code.
 * [2004.11.29.] ~ add LoginResult to handle message from PassengerListener. 
 * [2004.11.29.] ~ Change Networkpart to localhost to test. 
 * [2004.11.29./] ~ Make Taxicall form. close/open forms.
 * [2004.11.29.] ~ Change registerResult to selectResult.
 * [2004.11.29.] ~ Add comment to do in selectResult method.
 * [2004.11.30./] ~ Implementation select result method.
 */
public class PassengerClient 
{
	private String id;
	private NWClient server;
	//singleton to contact from other files.
	private static PassengerClient passenger = null;
	private Address serverAddress; 
	private static Login login;
	private static TaxiCall taxiCall;
	private Vector idList;
	private Vector passwdList;
	// TODO Setup Server Addressses.
	
	/**
	 * Constructor
	 * Setting up server address.
	 */
	public PassengerClient() 
	{
		id = null;
		server = new NWClient();
		server.start();
		try {
			InetAddress temp = InetAddress.getByName("127.0.0.1");
			serverAddress = new Address(temp,2230);
		}catch(UnknownHostException e) {
			System.out.println("Host Connection Failed");
			serverAddress = new Address();
			//TODO need implements to set it up in 127.0.0.1 or exit call.
		}
		
		makeUserList();
	}
	
	/**
	 * Singleton for accessing from other files.
	 * Make new singleton if there is none already exist.
	 *  
	 * @return snaptaxi.client.PassengerClient
	 */
	public static PassengerClient getPassenger() 
	{
		if (null == PassengerClient.passenger) {
	        PassengerClient.passenger = new PassengerClient();
	    }
		return PassengerClient.passenger;
	}
	
	/**
	 * Send user's id and passwd to server to login
	 * 
	 * @param id
	 * @param passwd
	 */
	public void login(String id, String passwd) 
	{
		this.id = id;
		//LoginMsg loginMSG = new LoginMsg(id, passwd);
		
		int num = idList.indexOf((Object)id);
		if(num == -1)
			loginResult(-1);
		else
		{
			String pass = (String)passwdList.get(num);
			if(pass.equals(passwd))
				loginResult(1);
			else
				loginResult(-2);
			//server.sendMSG(serverAddress,loginMSG);
		}
		
	}
	
	/**
	 * Receive LoginResultMsg and process the message.
	 * Close login form and open taxicall form, if login success.
	 * If login failed, display error message.
	 * 
	 * @param msg
	 */
	//public void loginResult(snaptaxi.LoginResultMsg msg)
	public void loginResult(int num)
	{
		/**
		 * LOGIN_ERROR				: error
		 * LOGIN_SUCCESS			: login succession
		 * LOGIN_UNREGISTER			: unresiter customer
		 * LOGIN_INVALID_PASS		: invalid password
		 */
		if(num == 1)//msg.isSuccess())
		{
			login.jLabel4.setText("Login Success!!"); 
			login.setVisible(false);
			// close login form and open taxi call form.
			taxiCall = new TaxiCall(id);
			taxiCall.setVisible(true);
		}
		else if(num == -2)//msg.isInvalidPass())
		{
			login.jLabel4.setText("Invalid Password"); 
			
		}
		else if(num == -1)//msg.isNotRegist())
		{
			login.jLabel4.setText("Unregisted ID."); // if success return 1
		}
		else
		{
			login.jLabel4.setText("Error occurs. plz try again."); // if success return 1
		}
	}
	
	/**
	 * Receive RegisterResultMsg and process the message.
	 * If regist successed, close login form and open taxicall form.
	 * If regist failed, display error message.
	 * 
	 * @param msg
	 */
	public void registResult(int num)//RegistResultMsg msg)
	{
		/**
		 * REGIST result state flag
		 * REGIST_ERROR				: error
		 * REGIST_SUCCESS			: regist succession
		 * REGIST_REGISTERED		: already registered customer
		 * REGIST_DBERROR			: database error.
		 */
		if(num == 1)//msg.isSuccess())
		{
			login.jLabel4.setText("Regist Success!!"); 
			login.setVisible(false);
			// close login form and open taxi call form.
			taxiCall = new TaxiCall(id);
			taxiCall.setVisible(true);
		}
		else if(num == -1) //msg.isRegistered())
		{
			login.jLabel4.setText("ID is already exist."); 
		}
		/*else if((msg.getRegistCode() == 1) || msg.getRegistCode() == 4)
		{
			login.jLabel4.setText("Error occurs. plz try again.");
		}*/
	}
	
	/**
	 * send user's id to server to logout.
	 * @param id
	 */
	public void logout(String id) 
	{
		//LogoutMsg logoutMSG = new LogoutMsg(id);
		//server.sendMSG(serverAddress,logoutMSG);
		// Close taxi call form and open login form.
		taxiCall.setVisible(false);
		login.setVisible(true);
		login.getJLabel4().setText("Logout Ǿϴ.");
	}
	
	/**
	 * Send user's location to call taxi.
	 * @param lotation(int x, int y)
	 */
	public void taxiCall(int posX, int posY) 
	{
		CallMsg call = new CallMsg(id, posX, posY);
		
		/*****************************************
		 * System.out.println("successtaxicall");
		 * taxiCall.successTaxiCall();
		 *****************************************/
		
		//TODO
		server.sendMSG(serverAddress,call);
		//TODO send MSG completed by Yoon Y.J. Needs testing.
	}
	
	/**
	 * Send user's id to server to cancel taxi call
	 * @param id
	 */
	public void cancelTaxiCall(String id) 
	{
		CancelCallMsg cancelCall = new CancelCallMsg(id);
		//TODO
		server.sendMSG(serverAddress,cancelCall);
		//TODO seng MSG completed by Yoon Y.J. Needs testing.
	}
	
	/**
	 * Send user's id and passwd to server to regist
	 * @param id
	 * @param passwd
	 */
	public void register(String id, String passwd) 
	{
		this.id = id;
		
		//RegistMsg registMSG = new RegistMsg(id, passwd, (byte)1);
		//server.sendMSG(serverAddress,registMSG);
		// TODO send MSG completed by Yoon Y.J. Needs testing.
		
		int num = idList.indexOf((Object)id);
		if(num != -1)
			registResult(-1);
		
		else
		{
			idList.add((Object)id);
			passwdList.add((Object)passwd);
			registResult(1);
		}
	}
	
	/**
	 * Receive SelectResultMsg and process the message.
	 * If regist successed, call successTaxiCall function in taxiCall.
	 * If regist failed, call failTaxiCall function in taxiCall.
	 * @param msg
	 */
	public void selectResult(SelectResultMsg msg) 
	{
		System.out.println(msg);
		if(msg.isSuccess())
		{
			taxiCall.successTaxiCall();	
			System.out.println("isSuccess");
		}
		else if(msg.getState()==2)
		{
			taxiCall.cancelConnect();
		}
		else if(msg.isFail())
		{
			taxiCall.failTaxiCall();
			System.out.println("isFailed");
		}
		else
		{
			taxiCall.errorOccured();
			System.out.println("error");
		} 
	}
	
	/**
	 * Send inregist message to server to unregist
	 * @param id
	 */
	
	public void unregister(String id) 
	{
		this.id = id;
		//UnregistMsg unregistMSG = new UnregistMsg(id);
		//server.sendMSG(serverAddress,unregistMSG);
		// TODO send MSG completed by Yoon Y.J. Needs testing.
		
		taxiCall.setVisible(false);
		login.setVisible(true);
		login.getJLabel4().setText("Ż Ǿϴ.");
	}
	
	/**
	 * Send message to client.
	 */
	public void sendMessage(Address address, Message msg)
	{
		this.server.sendMSG(address, msg);
	}
	
	public void makeUserList()
	{
		idList = new Vector(10);
		passwdList = new Vector(10);
		
		idList.add((Object)"test");
		passwdList.add((Object)"test");
		idList.add((Object)"biya");
		passwdList.add((Object)"tkfkdgo");
	}
	
	/**
	 * Main for test.
	 */
	public static void main(String[] args) {
		login = new Login();
		login.setVisible(true);
	}
}
