
package snaptaxi.taxi;

import snaptaxi.*;

import java.util.HashMap;

/**
 * Handle all message.
 * Actually this is main routine.
 * 
 * @author Korea Univ. SE Team #1
 * @version 1.0
 * @since 2004.11.20
 * [2004.11.20 /ȫ] ~ Class Design!
 * [2004.11.20 ] ~ Correct name of onSelectReult to onSelectResult-_-v
 * [2004.11.29 /ȫ] ~ Match to Singleton's method
 * [2004.11.29 /ȫ] ~ implement call mehtod
 * [2004.11.30 /ȫ] ~ implement loginResult, registerResult
 * [04.12.14 /ȫ] ~ Add more detail comment
 *
 */
public class TaxiListener 
{
	// Address -> ID Map for find ID from IPAddress.
	private HashMap addressToIDMap;

	// initialize listener.
	public TaxiListener() 
	{
		this.addressToIDMap = new HashMap();
	}

	/**
	 * When TaxiClient receive loginResult message from server
	 * Call TaxiClient's loginResult method
	 * 
	 * @param sender
	 * @param msg
	 */
	public void onLoginResult(Address sender, LoginResultMsg msg) 
	{
		/**
		 * Login result state flag
		 * LOGIN_ERROR				: error
		 * LOGIN_SUCCESS			: login succession
		 * LOGIN_UNREGISTER			: unresiter customer
		 * LOGIN_INVALID_PASS		: invalid password
		 */
		
		TaxiClient.getTaxi().loginResult(msg);		
	}
	
	/**
	 * When TaxiClient receive selectResult message from server
	 * Call TaxiClient's selectResult method
	 * 
	 * @param sender
	 * @param msg
	 */
	public void onSelectResult(Address sender, SelectResultMsg msg) 
	{
		/**
		 * Select passenger flag.
		 * RESULT_ERROR : error
		 * RESULT_SELECT_SUCCESS : select success
		 * RESULT_SELECT_FAIL : select fail.
		 * RESULT_SELECTED : passenger selected by other taxi.
		 * RESULT_UNSELECT : passenger is unselected.
		 */
		
		TaxiClient.getTaxi().selectResult();
	}
	
	/**
	 * When TaxiClient receive call message from server
	 * Call TaxiClient's call method
	 * 
	 * @param sender
	 * @param msg
	 */
	public void onCall(Address sender, CallMsg msg) 
	{
		/**
		 * @param passengerID
		 * @param posX
		 * @param posY
		 */
		
		TaxiClient.getTaxi().call(msg, sender.getPort());
	}

	/**
	 * When TaxiClient receive cancelCall message from server
	 * Call TaxiClient's cancelCall method
	 * 
	 * @param sender
	 * @param msg
	 */
	public void onCancelCall(Address sender, CancelCallMsg msg) 
	{
		/**
		 * passengerID  
		 */
		
		TaxiClient.getTaxi().cancelCall(msg);
	}
	
	/**
	 * When TaxiClient receive registResult message from server
	 * Call TaxiClient's registResult method
	 * 
	 * @param sender
	 * @param msg
	 */
	public void onRegistResult(Address sender, RegistResultMsg msg) 
	{
		/**
		 * REGIST result state flag
		 * REGIST_ERROR				: error
		 * REGIST_SUCCESS			: regist succession
		 * REGIST_REGISTERED		: already registered customer
		 * REGIST_DBERROR			: database error.
		 */
		
		TaxiClient.getTaxi().registResult(msg);
	}
	
	/**
	 * When TaxiClient receive gps position message from server
	 * 
	 * @param sender
	 * @param msg
	 */
	public void onGPSPosition(Address sender, GPSPositionMsg msg) 
	{
		/**
		 * id
		 * posX
		 * posY
		 */
		
	}

}